/*
Data: 2024-10-10
SOFTWARE DIMOSTRATIVO per l'interfacciamento a ai4call.

IL SOFTWARE VIENE FORNITO "COSÌ COM'È", SENZA GARANZIE DI ALCUN TIPO, ESPRESSE O IMPLICITE, INCLUSE, MA NON SOLO, LE GARANZIE DI COMMERCIABILITÀ, 
IN NESSUN CASO GLI AUTORI O I DETENTORI DEI DIRITTI D'AUTORE SARANNO RESPONSABILI DI EVENTUALI RECLAMI, DANNI O ALTRE RESPONSABILITÀ, SIA IN 
UN'AZIONE LEGALE CHE CONTRATTUALE, EXTRACONTRATTUALE O DI ALTRO TIPO, DERIVANTE DALL'UTILIZZO, ANCHE PARZIALE, DEL SOFTWARE.
*/

const express = require("express");
const bodyParser = require("body-parser");
const cors = require("cors");
const fetch = require("node-fetch");
const app = express();
const port = 3210;

// Configurazione di base per Express: abilitazione del supporto CORS e Body Parser per gestire i dati JSON di grandi dimensioni
app.use(cors());
app.use(bodyParser.json({ limit: "10gb" })); // Imposta un limite massimo per i payload JSON
app.use(bodyParser.urlencoded({ extended: false }));

// Log iniziale per indicare l'avvio del server
console.log(" ");
console.log(
  "-------------------------------------------------------------------------------------"
);
console.log(" ");
app.listen(port, () =>
  console.log(
    "progetto-ai4call (demo-whtsapp) in ascolto sulla porta: ",
    port,
    "\n"
  )
);

//---------------------------------------------------------------------------------------
// PRIMA REST DOVE ENTRERÀ LA CHIAMATA
// Questa è la prima rotta POST a cui si invia la chiamata iniziale.
// Questa è la FIRST URL che deve essere dichiarata sul progetto dal portale ai4call.com
// i numeri di telefono del chiamante e del destinatario, e si decide l'azione successiva
// in base al tipo di numero (mobile, fisso, estero, anonimo).
//---------------------------------------------------------------------------------------
let callControl = {};
app.post("/prima-richiesta", async (req, res) => {
  try {
    let Call_Id = req.headers["x-call-id"]; // Identificativo della chiamata passato nell'header
    console.log(
      "\n---------------------------------------------------------------------- "
    );
    console.log(
      "--------------------------START DEMO AI4CALL ----------------------------------- "
    );

    console.log("\nai4call ha chiamato mia API: 'prima-richiesta' con: \n");
    console.log("-- HEADER:", JSON.stringify(req.headers)); // Stampa gli header
    console.log("-- BODY:", JSON.stringify(req.body)); // Stampa il corpo della richiesta

    // Estrai i numeri di telefono del chiamante e del destinatario dal body della richiesta
    let { recipientNumber, callerNumber } = req.body;
    let sendresponse;

    // Chiamata API con gestione errori
    // Richiesta HTTP a una API REST esterna per ottenere messaggi o dati relativi al destinatario
    let datiAPI;
    try {
      datiAPI = await chiamaRestAPI(recipientNumber);
    } catch (apiError) {
      console.error("Errore durante la chiamata all'API:", apiError);
    }

    // Se l'API non restituisce dati validi
    if (!datiAPI) {
      sendresponse = {
        text: "IL numero non è attivo, arrivederci",
        action: "hangup",
      };
      return res.json(sendresponse);
    }

    // Destrutturazione sicura con valori predefiniti per evitare campi mancanti
    const {
      messaggi: {
        messaggio_no_mobile = "",
        messaggio_send_ws = "",
        link_ws,
      } = {},
    } = datiAPI;

    // Inizializza i dati della chiamata
    callControl[Call_Id] = {
      callerNumber,
      recipientNumber,
      messaggio_no_mobile,
      messaggio_send_ws,
      link_ws,
    };

    // Controlla il tipo di numero chiamante
    // Verifica il tipo di numero (mobile, fisso, estero, anonimo)
    let tipoNumero;
    try {
      tipoNumero = await checkNumberType(callerNumber);
    } catch (numberError) {
      console.error(
        "Errore durante il controllo del tipo di numero:",
        numberError
      );
      sendresponse = {
        text: "si è verificato un errore imprevisto, la preghiamo di riprovare più tardi, grazie",
        action: "hangup",
      };
    }
    //------------------------------------------------------------------------------------------
    // Logica di gestione del tipo di numero
    // Se il numero chiamante è un numero MOBILE inviamo un messaggio whatsapp
    // Se il numero chiamante è un numero ANONIMO/FISSO diamo messaggio audio
    // Se il numero chiamante è un numero ESTERO chiudiamo la chiamata
    //------------------------------------------------------------------------------------------
    switch (tipoNumero) {
      case "mobile_italiano":
        try {
          let response = await sendWhatsAppMessage(Call_Id);
          if (!response) {
            console.error("errore nell'invio del messaggio whatsapp ", error);
            // [ RESPONSE TO AI4CALL - ACTION HANGUP - riproduce testo tts e chiude la chiamata]
            sendresponse = {
              text: "si è verificato un errore imprevisto, la preghiamo di riprovare più tardi, grazie",
              action: "hangup",
            };
          } else {
            // [ RESPONSE TO AI4CALL - ACTION HANGUP - riproduce testo tts e chiude la chiamata - testo preso dalla rest esterna]
            sendresponse = {
              text: callControl[Call_Id].messaggio_send_ws,
              action: "hangup",
            };
          }
        } catch (error) {
          console.error("errore nell'invio del messaggio whatsapp ", error);
          // [ RESPONSE TO AI4CALL - ACTION HANGUP - riproduce testo tts e chiude la chiamata]
          sendresponse = {
            text: "si è verificato un errore imprevisto, la preghiamo di riprovare più tardi grazie",
            action: "hangup",
          };
        }
        break;

      case "fisso_italiano":
      case "anonymous":
        // [ RESPONSE TO AI4CALL - ACTION HANGUP - riproduce testo tts e chiude la chiamata]
        sendresponse = {
          text: callControl[Call_Id].messaggio_no_mobile,
          action: "hangup",
        };
        break;

      case "estero":
        // [ RESPONSE TO AI4CALL - ACTION HANGUP - chiude la chiamata]
        sendresponse = {
          text: "",
          action: "hangup",
        };
        break;

      default:
        // [ RESPONSE TO AI4CALL - ACTION HANGUP - riproduce testo tts e chiude la chiamata]
        sendresponse = {
          text: callControl[Call_Id].messaggio_no_mobile,
          action: "hangup",
        };
        break;
    }

    // INVIA LA RISPOSTA AD AI4CALL
    console.log(
      "\n-- =====> Io gli mando questo response:",
      JSON.stringify(sendresponse)
    );
    res.json(sendresponse);
  } catch (error) {
    console.error("Errore durante il processo generale:", error);
    sendresponse = {
      text: "IL numero non è attivo, arrivederci",
      action: "hangup",
    };
    return res.json(sendresponse);
  }
});

//---------------------------------------------------------------------------------------
// GESTIONE DELL'EVENTO HANGUP
// Questa è la HANGUP URL che deve essere dichiarata sul progetto dal portale ai4call.com
// Questa rotta viene chiamata da AI4Call quando una chiamata viene terminata (hangup).
//---------------------------------------------------------------------------------------

app.post("/hangup", async (req, res) => {
  let Call_Id = req.headers["x-call-id"];
  console.log(
    "\n---------------------------------------------------------------------- \n"
  );
  console.log("ai4call ha chiamato mia API: 'hangup' con:");
  console.log("-- BODY:", JSON.stringify(req.body));
  if (callControl[Call_Id]) {
    delete callControl[Call_Id];
    console.log(`Processo completato per Call_Id ${Call_Id}.`);
  } else {
    console.log(`Nessun dato trovato per Call_Id ${Call_Id}.`);
  }
  console.log(
    "\n----------- Processo completato con successo! ------------------------ \n"
  );
});

//---------------------------------------------------------------------------------------
// Funzione per verificare il tipo di numero (mobile, fisso, estero, anonimo)
// Questa funzione analizza il numero di telefono per determinare il tipo e gestirlo
// di conseguenza.
//---------------------------------------------------------------------------------------
async function checkNumberType(number) {
  try {
    if (number.startsWith("+39")) {
      number = number.slice(3); // Rimuovi il prefisso internazionale per numeri italiani
    }

    if (number.startsWith("3")) {
      return "mobile_italiano"; // Numeri mobili iniziano con 3 in Italia
    } else if (number.startsWith("0")) {
      return "fisso_italiano"; // Numeri fissi iniziano con 0
    } else if (number.startsWith("+")) {
      return "estero"; // Numeri esteri hanno il prefisso +
    } else if (number === "anonymous") {
      return "anonymous"; // Gestione numeri anonimi
    } else {
      return "numero_sconosciuto"; // Numeri non riconosciuti
    }
  } catch (err) {
    console.error("Errore nel check del numero:", err);
  }
}

//---------------------------------------------------------------------------------------
// Funzione per inviare un messaggio WhatsApp
// Questa funzione utilizza l'API di WhatsApp per inviare un messaggio al numero di
// telefono specificato. Usa l'access token per autenticarsi e inviare il messaggio.
//---------------------------------------------------------------------------------------
async function sendWhatsAppMessage(Call_Id) {
  const response = await fetch(
    "https://graph.facebook.com/v16.0/[QUI INSERIRE L'ENDPOINT RILASCIATO DA META]/messages",
    {
      method: "POST",
      headers: {
        Authorization: `Bearer [QUI INSERIRE IL TOKEN RILASCIATO DA META]`,
        "Content-Type": "application/json",
      },
      body: JSON.stringify({
        messaging_product: "whatsapp",
        to: callControl[Call_Id].callerNumber, // Numero di telefono del destinatario
        type: "template",
        template: {
          name: "[INSERIRE IL NOME DEL TEMPLATE DA USARE]",
          language: {
            code: "it", // Lingua del template
          },
          components: [
            {
              type: "body",
              parameters: [
                {
                  type: "text",
                  text: callControl[Call_Id].link_ws, // Inserisci qui il link dinamico
                },
              ],
            },
          ],
        },
      }),
    }
  );

  // Controlla se la risposta è positiva
  if (response.ok) {
    const data = await response.json();
    console.log("Messaggio WhatsApp inviato con successo:");
    return true;
  } else {
    const errorData = await response.json();
    console.error("Errore durante l'invio del messaggio WhatsApp:", errorData);
    return false;
  }
}

//---------------------------------------------------------------------------------------
// Funzione realizzata come esempio per una chiamata all'API REST esterna a questo progetto
// Questa funzione effettua una richiesta HTTP a una API REST esterna per ottenere
// messaggi o dati relativi al destinatario. Restituisce il JSON ottenuto dalla richiesta.
//---------------------------------------------------------------------------------------
async function chiamaRestAPI(recipientNumber) {
  try {
    const url = `http://localhost:[porta]/dati/${recipientNumber}`;
    const response = await fetch(url);

    if (!response.ok) {
      throw new Error(`Errore nella richiesta: ${response.status}`);
    }

    const data = await response.json(); // Estrai i dati dal corpo della risposta
    return data;
  } catch (error) {
    console.error("Errore durante la chiamata alla REST API:", error.message);
  }
}
